#####################################################################
#   Buffer input data for uncertainty calculation (5nm for logbook data, .5 nautical mile
#   buffer for others, skip 2 overly complex shapefiles that when buffered don't impact 
#   the output)
#
#
#   By: Colleen Sullivan
#   Date: 6/2012
#
#####################################################################

# All of the code occurs in a try/except block to help catch errors
try:
     print("Start")
          
     # Shapefiles to buffer
     ClippedFolder = "Z:/MMS/Sullivan/Thesis/Data/Clipped/"
     BufferedFolder = "Z:/MMS/Sullivan/Thesis/Data/BufferedRasters/Input_Buff/"

     # Import the module containing the ArcGIS Interface classes
     import ArcGISInterfaceModule
     
     # Import the operating system module to allow file/folder creation and loops later
     import os
     
     # Remove results of the last run before beginning, if they exist, and either way
     # create folders to store results of this run
     import shutil
     
     if os.path.exists(BufferedFolder):
          shutil.rmtree(BufferedFolder)
     os.makedirs(BufferedFolder)
     
     # A log is kept of processing progress, any errors returned will be saved to this
     # log, first the start time of processing is
     # recorded:
     TheTextFile=open("Z:/MMS/Sullivan/Thesis/Data/"+"BufferingInputDataLog.txt","w")
     import datetime
     now = datetime.datetime.now()
     timestamp = format(now.month) + "/" + format(now.day) + "/" + format(now.year) + " " + format(now.hour) + ":" + format(now.minute) + ":" + format(now.second)
     TheTextFile.write("Beginning processing: " + timestamp + "\n")
     
     # To batch process, the code must recongize all of the subfolders of the top level 
     # folder (each is its own data category). So, a list of folders is generated and a 
     # loop is used to run the code for each shapefile found in each folder. This code 
     # was adapted from that on Jim Graham's course website: 
     # http://ibis.colostate.edu/DH.php?WC=/WS/Jim/Geo599/08_5_BatchProcessinWithArcGIS.html
     TheList=os.listdir(ClippedFolder)
     for TheFolder in TheList:
          
          # Create a subfolder in the folder 'Input_Buff' for this category to store the
          # buffered files
          BufferedSubfolder = BufferedFolder+TheFolder+"/"
          os.makedirs(BufferedSubfolder)          
          
          # Set the processing path
          ProcessingPath = ClippedFolder+TheFolder+"/"
          
          TheTextFile.write("\nNow processing shapefiles in path:" + ProcessingPath + "\n")
          print("Now processing shapefiles in path:" + ProcessingPath + "\n")
          
          # ~~ The Analysis Tools class is accessed ~~
          TheInterface=ArcGISInterfaceModule.ATInterface()
          
          TheProcessingList=os.listdir(ProcessingPath)
          for TheFile in TheProcessingList:
                    
               # The file name is split from its extension for use in the commands that
               # follow and to determine if it is a .shp, each shapefile name appears
               # multiple times in the list because of its component files, so this
               # ensures each shapefile is only actually processed once.
               TheFileName, TheFileExtension = os.path.splitext(TheFile)
               if (TheFileExtension==".shp"):
                    TheTextFile.write("  - Shapefile: " + TheFileName + " -\n")
                    
                    filestring = str(TheFileName)
                    if filestring.startswith("CDFG") or filestring.startswith("cdfg") or filestring.startswith("ODFW") or filestring.startswith("PacFIN") == True:
                         # Logbook data is buffered by 5 nm
                         TheInterface.Buffer(ProcessingPath+TheFile, BufferedSubfolder+TheFile, "5 NauticalMiles")
                         TheTextFile.write("     Buffered by 5 nm\n")
                    
                    elif TheFileName == "seagrass_pub" or TheFileName == "altb04":
                         # These two habitat shapefiles are too complex to buffer by 
                         # .5 nm, and because they don't impact the output buffered 
                         # raster for the category, they are simply copied to the 
                         # buffered input folder without buffering first
                         
                         # ~~ The Data Management Tools class is accessed ~~
                         TheInterface = ArcGISInterfaceModule.DMInterface()
                         
                         # Copy Shapefile is executed
                         TheInterface.CopyShapefile(ProcessingPath+TheFile, BufferedSubfolder+TheFile)
                         TheTextFile.write("     Copied without buffering\n")
                         
                         # ~~ The Analysis Tools class is accessed so that the other two
                         # options in this loop still work ~~
                         TheInterface=ArcGISInterfaceModule.ATInterface()
                         
                    else:
                         # Other data is buffered by .5 nm
                         TheInterface.Buffer(ProcessingPath+TheFile, BufferedSubfolder+TheFile, "0.5 NauticalMiles")
                         TheTextFile.write("     Buffered by .5 nm\n")

# In case of error, the code will break to here and return the error message to the text
# file
except Exception, err:
     print("*** Error. Stopped." + str(err))
     TheTextFile.write(str(err)+"\n")

# A final message is printed with the end processing time and the text file log is closed.
now = datetime.datetime.now()
timestamp = format(now.month) + "/" + format(now.day) + "/" + format(now.year) + " " + format(now.hour) + ":" + format(now.minute) + ":" + format(now.second)
TheTextFile.write("\nCompleted Script: " + timestamp)
TheTextFile.close
print("End")