#####################################################################
#   Carry out potential for conflict map algebra
#
#
#   By: Colleen Sullivan
#   Date: 6/2012
#
#####################################################################

# All of the code occurs in a try/except block to help catch errors
try:
     print("Start")
     
     # Import the module containing the ArcGIS Interface classes so we can use its tools
     # and get parameters in the next step
     import ArcGISInterfaceModule
     
     # Obtain the user inputs and set as parameters:
     # Most likely "Z:/MMS/Sullivan/Thesis/Data/Rasters/"     
     TheOriginalFolder = arcpy.GetParameterAsText(0) + "/"
     # Name for the output raster and folder to save the weighted versions of the 
     # rasters     
     TheOutputName = arcpy.GetParameterAsText(1)          
     # Path in which to create the aforementioned folder     
     TheOutputFolderPath = arcpy.GetParameterAsText(2)         
     
     if len(TheOutputName) > 13:
          TheOutputName = TheOutputName[0:13]
     
     # Import the operating system module to allow file/folder creation and loops later
     import os
     
     # Remove results of the last run before beginning, if they exist, and either way
     # create folders to store results of this run
     import shutil
     
     TheOutputFolder = TheOutputFolderPath + "/" + TheOutputName + "/"
     if os.path.exists(TheOutputFolder):
          shutil.rmtree(TheOutputFolder)
          
     # Store the user inputs for weights as parameters to use in map algebra
     # Archeological
     Weight_wrecks = arcpy.GetParameterAsText(3)
     # Area of special concern
     Weight_habitat = arcpy.GetParameterAsText(4)
     Weight_protected = arcpy.GetParameterAsText(5)
     Weight_tribal = arcpy.GetParameterAsText(6)
     # Research - sampling locations
     Weight_research = arcpy.GetParameterAsText(7)
     # Military
     Weight_military = arcpy.GetParameterAsText(8)
     # Sand/Gravel Source and Disposal
     Weight_disposaldump = arcpy.GetParameterAsText(9)
     Weight_dredge = arcpy.GetParameterAsText(10)
     # Oil and gas deposits and infrastructure/cables
     Weight_cable = arcpy.GetParameterAsText(11)
     Weight_pipeline = arcpy.GetParameterAsText(12)
     # Recreation activity
     Weight_rec_boating = arcpy.GetParameterAsText(13)
     Weight_rec_fishing = arcpy.GetParameterAsText(14)
     Weight_rec_wildlife = arcpy.GetParameterAsText(15)
     Weight_rec_other = arcpy.GetParameterAsText(16)
     # Commercial fishing
     Weight_fish_closure = arcpy.GetParameterAsText(17)
     Weight_fish_line = arcpy.GetParameterAsText(18)
     Weight_fish_pots = arcpy.GetParameterAsText(19)
     Weight_fish_traps = arcpy.GetParameterAsText(20)
     Weight_fish_trawl = arcpy.GetParameterAsText(21)
     Weight_fish_troll = arcpy.GetParameterAsText(22)
     Weight_fish_other = arcpy.GetParameterAsText(23)
     # Marine transportation
     Weight_marinetrans_1 = arcpy.GetParameterAsText(24)
     Weight_marinetrans_2 = arcpy.GetParameterAsText(25)
     Weight_marinetrans_3 = arcpy.GetParameterAsText(26)
     Weight_marinetrans_4 = arcpy.GetParameterAsText(27)
     Weight_navaid = arcpy.GetParameterAsText(28)
     
     time.sleep(0.001)
     os.makedirs(TheOutputFolder)
     
     # A log is kept of processing progress, any errors returned will be saved to this
     # log, first the start time of processing is recorded:
     TheTextFile=open(TheOutputFolder+"MapAlgebraLog.txt","w")
     import datetime
     now = datetime.datetime.now()
     timestamp = format(now.month) + "/" + format(now.day) + "/" + format(now.year) + " " + format(now.hour) + ":" + format(now.minute) + ":" + format(now.second)
     TheTextFile.write("Beginning processing: " + timestamp + "\n")
     
     # ---------- PART 1: Weight each raster and save the weighted raster ----------
     # ~~ The ArcPy class is accessed ~~
     TheInterface = ArcGISInterfaceModule.ArcPyClasses()
     
     # The list of category rasters is obtained
     TheList = TheInterface.GetRasterList(TheOriginalFolder, "*")
     
     # ~~ The Spatial Analyst Tools class is accessed ~~
     TheInterface = ArcGISInterfaceModule.SAInterface()
     
     # An if statement is used to weight each raster with the appropriate user-inputed
     # weight
     for TheFile in TheList:
          if TheFile == "wrecks":
               TheWeight = Weight_wrecks
          elif TheFile == "habitat":
               TheWeight = Weight_habitat
          elif TheFile == "protected":
               TheWeight = Weight_protected
          elif TheFile == "tribal":
               TheWeight = Weight_tribal
          elif TheFile == "research":
               TheWeight = Weight_research
          elif TheFile == "military":
               TheWeight = Weight_military
          elif TheFile == "disposaldump":
               TheWeight = Weight_disposaldump
          elif TheFile == "dredge":
               TheWeight = Weight_dredge
          elif TheFile == "cable":
               TheWeight = Weight_cable
          elif TheFile == "pipeline":
               TheWeight = Weight_pipeline
          elif TheFile == "rec_boating":
               TheWeight = Weight_rec_boating
          elif TheFile == "rec_fishing":
               TheWeight = Weight_rec_fishing
          elif TheFile == "rec_wildlife":
               TheWeight = Weight_rec_wildlife
          elif TheFile == "rec_other":
               TheWeight = Weight_rec_other
          elif TheFile == "fish_closure":
               TheWeight = Weight_fish_closure
          elif TheFile == "fish_line":
               TheWeight = Weight_fish_line
          elif TheFile == "fish_other":
               TheWeight = Weight_fish_other
          elif TheFile == "fish_pots":
               TheWeight = Weight_fish_pots
          elif TheFile == "fish_traps":
               TheWeight = Weight_fish_traps
          elif TheFile == "fish_trawl":
               TheWeight = Weight_fish_trawl
          elif TheFile == "fish_troll":
               TheWeight = Weight_fish_troll
          elif TheFile == "marinetrans_1":
               TheWeight = Weight_marinetrans_1
          elif TheFile == "marinetrans_2":
               TheWeight = Weight_marinetrans_2
          elif TheFile == "marinetrans_3":
               TheWeight = Weight_marinetrans_3
          elif TheFile == "marinetrans_4":
               TheWeight = Weight_marinetrans_4
          elif TheFile == "navaid":
               TheWeight = Weight_navaid
     
          # The file and weight are passed to derive an output weighted raster
          temp = TheInterface.MultMapAlgebra(TheOriginalFolder+TheFile, TheWeight)
          temp.save(TheOutputFolder+TheFile)
          TheTextFile.write("Weighted version generated of:"+TheFile+"\n")
     
     # ---------- PART 2: Add up the weighted rasters: ----------
    
     # ~~ The Spatial Analyst Tools class is accessed ~~
     TheInterface = ArcGISInterfaceModule.SAInterface()
     
     # An expression is generated and evaluated to add the rasters together
     temp = TheInterface.WeightedOverlayAddition(TheOutputFolder)
     temp.save(TheOutputFolder+TheOutputName)
     TheTextFile.write("Map algebra complete, final raster generated\n")

# In case of error, the code will break to here and return the error message to the text
# file
except Exception, err:
     print("Error. Stopped.")
     TheTextFile.write(str(err)+"\n")

# A final message is printed with the end processing time and the text file log is closed.
now = datetime.datetime.now()
timestamp = format(now.month) + "/" + format(now.day) + "/" + format(now.year) + " " + format(now.hour) + ":" + format(now.minute) + ":" + format(now.second)
TheTextFile.write("Completed Script: " + timestamp)
TheTextFile.close
print("End")